
package com.mortals.xhx.module.role.service.impl;

import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;

import com.mortals.xhx.module.menu.model.MenuEntity;
import com.mortals.xhx.module.menu.model.MenuQuery;
import com.mortals.xhx.module.menu.service.MenuService;
import com.mortals.xhx.module.role.dao.RoleAuthDao;
import com.mortals.xhx.module.role.model.*;
import com.mortals.xhx.module.role.service.RoleAuthService;
import com.mortals.xhx.module.role.service.RoleModelService;
import com.mortals.xhx.module.role.service.RoleUserService;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import java.util.*;
import java.util.stream.Collectors;

@Service("roleAuthService")
public class RoleAuthServiceImpl extends AbstractCRUDServiceImpl<RoleAuthDao, RoleAuthEntity, Long> implements RoleAuthService {

    @Autowired
    private MenuService menuService;
    @Autowired
    private RoleModelService roleModelService;
    @Autowired
    private RoleUserService roleUserService;

    @Override
    public void assignMenu(RoleAuthQuery query) {
        Map<Long, MenuEntity> menuEntityMap = menuService.find(new MenuQuery()).stream().collect(Collectors.toMap(x -> x.getId(), y -> y));
        // 删除角色菜单资源老数据
        Long roleId = query.getRoleId();
        Map<String, Object> condition = new HashMap<>();
        condition.put("roleId", roleId);
        this.dao.delete(condition);

        Set<Long> curSet = query.getMenuIdList().stream().collect(Collectors.toSet());
        query.getMenuIdList().stream().forEach(item -> {
            buildMenus(curSet, menuEntityMap.get(item), menuEntityMap);
        });
        List<RoleAuthEntity> list = curSet.stream().map(item -> {
            RoleAuthEntity entity = new RoleAuthEntity();
            entity.setRoleId(roleId);
            entity.setMenuId(item);
            return entity;
        }).collect(Collectors.toList());

        this.save(list);
        roleModelService.deleteByRoleId(roleId);
        RoleModelEntity update = new RoleModelEntity();
        update.setRoleId(roleId);
        update.setModelIds(query.getModelIds());
        update.setCensusIds(query.getCensusIds());
        roleModelService.save(update);
    }

    private void buildMenus(Set<Long> curSet, MenuEntity menuEntity, Map<Long, MenuEntity> menuEntityMap) {
        if (!ObjectUtils.isEmpty(menuEntity)) {
            if (!curSet.contains(menuEntity.getId())) {
                curSet.add(menuEntity.getId());
            }
            if (!ObjectUtils.isEmpty(menuEntity.getParentId())) {
                buildMenus(curSet, menuEntityMap.get(menuEntity.getParentId()), menuEntityMap);
            }
        }
    }

    @Override
    public Map<Long,List<Long>> getMenuIdByRole(Long roleId) {
        RoleAuthQuery condition = new RoleAuthQuery();
        condition.setRoleId(roleId);
        List<RoleAuthEntity> roleModelEntities = this.find(condition);
        if(CollectionUtils.isNotEmpty(roleModelEntities)) {
            List<Long> menuIds = roleModelEntities.stream().map(RoleAuthEntity::getMenuId).collect(Collectors.toList());
            MenuQuery query1 = new MenuQuery();
            query1.setIdList(menuIds);
            List<MenuEntity> menuEntities = menuService.find(query1);
            Map<Long,List<Long>> map = new HashMap<>();
            menuEntities.forEach(m->{
                if(map.containsKey(m.getParentId())){
                    map.get(m.getParentId()).add(m.getId());
                }else {
                    List<Long> menuIdList = new ArrayList<>();
                    menuIdList.add(m.getId());
                    map.put(m.getParentId(),menuIdList);
                }
            });
            return map;
        }else {
            return Collections.emptyMap();
        }
    }


}